@@ -39,6 +39,7 @@ os._mkdir    = os._mkdir or os.mkdir
 os._rmdir    = os._rmdir or os.rmdir
 os._touch    = os._touch or os.touch
 os._tmpdir   = os._tmpdir or os.tmpdir
+os._fscase   = os._fscase or os.fscase
 os._setenv   = os._setenv or os.setenv
 os._getenvs  = os._getenvs or os.getenvs
 os._cpuinfo  = os._cpuinfo or os.cpuinfo
@@ -54,6 +55,7 @@ os.SYSERR_NOT_ACCESS  = 3
 
 -- copy single file or directory
 function os._cp(src, dst, rootdir, opt)
+    opt = opt or {}
     assert(src and dst)
 
     -- reserve the source directory structure if opt.rootdir is given
@@ -67,7 +69,7 @@ function os._cp(src, dst, rootdir, opt)
     end
 
     -- is file or link?
-    local symlink = opt and opt.symlink
+    local symlink = opt.symlink
     if os.isfile(src) or (symlink and os.islink(src)) then
 
         -- the destination is directory? append the filename
@@ -80,6 +82,9 @@ function os._cp(src, dst, rootdir, opt)
         end
 
         -- copy or link file
+        if opt.force and os.isfile(dst) then
+            os.rmfile(dst)
+        end
         if not os.cpfile(src, dst, symlink) then
             local errors = os.strerror()
             if symlink and os.islink(src) then
@@ -105,8 +110,6 @@ function os._cp(src, dst, rootdir, opt)
         if not os.cpdir(src, dst, symlink) then
             return false, string.format("cannot copy directory %s to %s,  %s", src, dst, os.strerror())
         end
-
-    -- not exists?
     else
         return false, string.format("cannot copy file %s, file not found!", src)
     end
@@ -114,10 +117,10 @@ function os._cp(src, dst, rootdir, opt)
 end
 
 -- move single file or directory
-function os._mv(src, dst)
+function os._mv(src, dst, opt)
+    opt = opt or {}
     assert(src and dst)
 
-    -- exists file or directory?
     if os.exists(src) then
 
         -- the destination directory exists? append the filename
@@ -126,10 +129,12 @@ function os._mv(src, dst)
         end
 
         -- move file or directory
+        if opt.force and os.isfile(dst) then
+            os.rmfile(dst)
+        end
         if not os.rename(src, dst) then
             return false, string.format("cannot move %s to %s %s", src, dst, os.strerror())
         end
-    -- not exists?
     else
         return false, string.format("cannot move %s to %s, file %s not found!", src, dst, os.strerror())
     end
@@ -434,7 +439,7 @@ function os.cp(srcpath, dstpath, opt)
 end
 
 -- move files or directories
-function os.mv(srcpath, dstpath)
+function os.mv(srcpath, dstpath, opt)
 
     -- check arguments
     if not srcpath or not dstpath then
@@ -446,10 +451,10 @@ function os.mv(srcpath, dstpath)
     dstpath = tostring(dstpath)
     local srcpathes = os._match_wildcard_pathes(srcpath)
     if type(srcpathes) == "string" then
-        return os._mv(srcpathes, dstpath)
+        return os._mv(srcpathes, dstpath, opt)
     else
         for _, _srcpath in ipairs(srcpathes) do
-            local ok, errors = os._mv(_srcpath, dstpath)
+            local ok, errors = os._mv(_srcpath, dstpath, opt)
             if not ok then
                 return false, errors
             end
@@ -483,9 +488,13 @@ function os.rm(filepath)
 end
 
 -- link file or directory to the new symfile
-function os.ln(srcpath, dstpath)
+function os.ln(srcpath, dstpath, opt)
+    opt = opt or {}
     srcpath = tostring(srcpath)
     dstpath = tostring(dstpath)
+    if opt.force and os.isfile(dstpath) then
+        os.rmfile(dstpath)
+    end
     if not os.link(srcpath, dstpath) then
         return false, string.format("cannot link %s to %s, %s", srcpath, dstpath, os.strerror())
     end
@@ -770,7 +779,7 @@ function os.execv(program, argv, opt)
         local head = file:read("l")
         if head and head:startswith("#!") then
             -- we cannot run `/bin/sh` directly on msys2/cygwin
-            -- because `/bin/sh` is not real file path, maybe we need convert it.
+            -- because `/bin/sh` is not real file path, maybe we need to convert it.
             local subhost = os.subhost()
             if subhost == "msys" or subhost == "cygwin" then
                 filename = "sh"
@@ -1068,8 +1077,24 @@ function os.isroot()
 end
 
 -- is case-insensitive filesystem?
-function os.fscase()
-    if os._FSCASE == nil then
+function os.fscase(filepath)
+    if os._FSCASE == nil or filepath then
+        if os._fscase then
+            if filepath then
+                assert(os.exists(filepath), filepath .. " not found in os.fscase()")
+            else
+                local tmpdir = os.tmpdir()
+                if not os.isdir(tmpdir) then
+                    os.mkdir(tmpdir)
+                end
+                filepath = tmpdir
+            end
+            local fscase = os._fscase(filepath)
+            if fscase ~= -1 then
+                os._FSCASE = (fscase == 1)
+                return os._FSCASE
+            end
+        end
         if os.host() == "windows" then
             os._FSCASE = false
         else
@@ -1118,7 +1143,9 @@ function os.getenvs()
         local p = line:find('=', 1, true)
         if p then
             local key = line:sub(1, p - 1):trim()
-            if os.host() == "windows" then
+            -- only translate Path to PATH on windows
+            -- @see https://github.com/xmake-io/xmake/issues/3752
+            if os.host() == "windows" and key:lower() == "path" then
                 key = key:upper()
             end
             local values = line:sub(p + 1):trim()
@@ -1379,3 +1406,4 @@ end
 
 -- return module
 return os
+
