@@ -116,7 +116,7 @@ function nf_symbols(self, levels, target)
 
             -- check and add symbol output file
             --
-            -- @note we need use `{}` to wrap it to avoid expand it
+            -- @note we need to use `{}` to wrap it to avoid expand it
             -- https://github.com/xmake-io/xmake/issues/2061#issuecomment-1042590085
             local pdbflags = {"-Fd" .. (target:is_static() and symbolfile or path.join(symboldir, "compile." .. path.filename(symbolfile)))}
             if self:has_flags({"-FS", "-Fd" .. os.nuldev() .. ".pdb"}, "cxflags", { flagskey = "-FS -Fd" }) then
@@ -163,7 +163,7 @@ function nf_optimize(self, level)
         none        = "-Od"
     ,   faster      = "-Ox"
     ,   fastest     = "-O2 -fp:fast"
-    ,   smallest    = "-O1 -GL" -- /GL and (/OPT:REF is on by default in linker), we need enable /ltcg
+    ,   smallest    = "-O1 -GL" -- /GL and (/OPT:REF is on by default in linker), we need to enable /ltcg
     ,   aggressive  = "-O2 -fp:fast"
     }
     return maps[level]
@@ -180,10 +180,14 @@ end
 function nf_vectorext(self, extension)
     local maps =
     {
-        sse    = "-arch:SSE"
-    ,   sse2   = "-arch:SSE2"
-    ,   avx    = "-arch:AVX"
-    ,   avx2   = "-arch:AVX2"
+        sse        = "-arch:SSE"
+    ,   sse2       = "-arch:SSE2"
+    ,   ["sse4.2"] = "/d2archSSE42" -- the only undocumented way works, @see https://github.com/xmake-io/xmake/issues/3786
+    ,   avx        = "-arch:AVX"
+    ,   avx2       = "-arch:AVX2"
+    ,   avx512     = "-arch:AVX512" -- for msvc 2019+ avx512 support
+    ,   fma        = "-arch:AVX2"
+    ,   all        = {"-arch:SSE", "-arch:SSE2", "/d2archSSE42", "-arch:AVX", "-arch:AVX2", "-arch:AVX512"}
     }
     local flag = maps[extension]
     if flag and self:has_flags(flag, "cxflags") then
@@ -277,6 +281,14 @@ function nf_includedir(self, dir)
     return {"-I" .. path.translate(dir)}
 end
 
+-- make the force include flag
+function nf_forceinclude(self, headerfile, target)
+    local sourcekinds = target and target:extraconf("forceincludes", headerfile, "sourcekinds")
+    if not sourcekinds or table.contains(table.wrap(sourcekinds), self:kind()) then
+        return {"-FI", headerfile}
+    end
+end
+
 -- make the sysincludedir flag
 function nf_sysincludedir(self, dir)
     local has_external_includedir = _g._HAS_EXTERNAL_INCLUDEDIR
@@ -311,13 +323,48 @@ function nf_exception(self, exp)
     return maps[exp]
 end
 
+-- make the encoding flag
+-- @see https://github.com/xmake-io/xmake/issues/2471
+--
+-- e.g.
+-- set_encodings("utf-8")
+-- set_encodings("source:utf-8", "target:utf-8")
+function nf_encoding(self, encoding)
+    local kind
+    local charset
+    local splitinfo = encoding:split(":")
+    if #splitinfo > 1 then
+        kind = splitinfo[1]
+        charset = splitinfo[2]
+    else
+        charset = encoding
+    end
+    local charsets = {
+        ["utf-8"] = "utf-8",
+        utf8 = "utf-8"
+    }
+    local flags = {}
+    charset = charsets[charset:lower()]
+    if charset then
+        if not kind and charset == "utf-8" then
+            table.insert(flags, "/utf-8")
+        else
+            if kind == "source" or not kind then
+                table.insert(flags, "-source-charset=" .. charset)
+            end
+            if kind == "target" or not kind then
+                table.insert(flags, "-execution-charset=" .. charset)
+            end
+        end
+    end
+    if #flags > 0 then
+        return flags
+    end
+end
+
 -- make the c precompiled header flag
 function nf_pcheader(self, pcheaderfile, target)
-
-    -- for c source file
     if self:kind() == "cc" then
-
-        -- patch objectfile
         local objectfiles = target:objectfiles()
         if objectfiles then
             table.insert(objectfiles, target:pcoutputfile("c") .. ".obj")
@@ -328,11 +375,7 @@ end
 
 -- make the c++ precompiled header flag
 function nf_pcxxheader(self, pcheaderfile, target)
-
-    -- for c++ source file
     if self:kind() == "cxx" then
-
-        -- patch objectfile
         local objectfiles = target:objectfiles()
         if objectfiles then
             table.insert(objectfiles, target:pcoutputfile("cxx") .. ".obj")
@@ -528,7 +571,7 @@ end
 -- compile preprocessed file
 function _compile_preprocessed_file(program, cppinfo, opt)
     local outdata, errdata = vstool.iorunv(program, winos.cmdargv(table.join(cppinfo.cppflags, "-Fo" .. cppinfo.objectfile, cppinfo.cppfile)), opt)
-    -- we need get warning information from output
+    -- we need to get warning information from output
     cppinfo.outdata = outdata
     cppinfo.errdata = errdata
 end
@@ -599,7 +642,7 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
                 end
             end
 
-            -- we need show full file path to goto error position if xmake is called in vstudio
+            -- we need to show full file path to goto error position if xmake is called in vstudio
             -- https://github.com/xmake-io/xmake/issues/1049
             if _is_in_vstudio() then
                 if compflags == flags then
