@@ -371,7 +371,7 @@ end
 -- install packages
 function _install_packages(packages_install, packages_download, installdeps)
 
-    -- we need hide wait characters if is not a tty
+    -- we need to hide wait characters if is not a tty
     local show_wait = io.isatty()
 
     -- init installed packages
@@ -478,8 +478,8 @@ function _install_packages(packages_install, packages_download, installdeps)
                 if downloaded then
                     if not action_install(instance) then
                         assert(instance:is_precompiled(), "package(%s) should be precompiled", instance:name())
-                        -- we need disable built and re-download and re-install it
-                        instance:fackback_build()
+                        -- we need to disable built and re-download and re-install it
+                        instance:fallback_build()
                         action_download(instance)
                         action_install(instance)
                     end
@@ -537,6 +537,11 @@ function _install_packages(packages_install, packages_download, installdeps)
                 table.insert(downloading, instance:displayname())
             end
         end
+        -- we just return it directly if no thing is waited
+        -- @see https://github.com/xmake-io/xmake/issues/3535
+        if #installing == 0 and #downloading == 0 then
+            return
+        end
 
         -- get waitobjs tips
         local tips = nil
@@ -623,7 +628,7 @@ function _get_package_installdeps(packages)
     local installdeps = {}
     local packagesmap = {}
     for _, instance in ipairs(packages) do
-        -- we need use alias name first for toolchain/packages
+        -- we need to use alias name first for toolchain/packages
         packagesmap[instance:alias() or instance:name()] = instance
     end
     for _, instance in ipairs(packages) do
@@ -631,7 +636,7 @@ function _get_package_installdeps(packages)
         if instance:orderdeps() then
             deps = table.copy(instance:orderdeps())
         end
-        -- patch toolchain/packages to installdeps, because we need install toolchain package first
+        -- patch toolchain/packages to installdeps, because we need to install toolchain package first
         for _, toolchain in ipairs(instance:toolchains()) do
             for _, packagename in ipairs(toolchain:config("packages")) do
                 if packagesmap[packagename] ~= instance then -- avoid loop recursion
@@ -761,7 +766,7 @@ function main(requires, opt)
         print("upgrading packages ..")
     end
 
-    -- some packages are modified? we need fix packages list and all deps
+    -- some packages are modified? we need to fix packages list and all deps
     if packages_modified then
         order_packages = {}
         _replace_packages(packages, packages_modified)
