@@ -144,7 +144,9 @@ function _load_require(require_str, requires_extra, parentinfo)
     end
 
     -- get required building configurations
-    local require_build_configs = require_extra.configs or require_extra.config
+    -- we need to clone a new configs object, because the whole requireinfo will be modified later.
+    -- @see https://github.com/xmake-io/xmake-repo/pull/2067
+    local require_build_configs = table.clone(require_extra.configs or require_extra.config)
     if require_extra.debug then
         require_build_configs = require_build_configs or {}
         require_build_configs.debug = true
@@ -153,7 +155,7 @@ function _load_require(require_str, requires_extra, parentinfo)
     -- require packge in the current host platform
     if require_extra.host then
         if is_subhost(core_package.targetplat()) and os.subarch() == core_package.targetarch() then
-            -- we need pass plat/arch to avoid repeat installation
+            -- we need to pass plat/arch to avoid repeat installation
             -- @see https://github.com/xmake-io/xmake/issues/1579
         else
             require_extra.plat = os.subhost()
@@ -167,11 +169,17 @@ function _load_require(require_str, requires_extra, parentinfo)
     "verify", "external", "private", "build", "configs", "version")
     for name, value in pairs(require_extra) do
         if not extra_options:has(name) then
-            print("not found")
             wprint("add_requires(\"%s\") has unknown option: {%s=%s}!", require_str, name, tostring(value))
         end
     end
 
+    -- we always use xmake package, `add_requires("xmake::zlib")`,
+    -- it is equivalent to `add_requires("zlib", {system = false})`
+    if packagename:startswith("xmake::") then
+        packagename = packagename:sub(8)
+        require_extra.system = false
+    end
+
     -- init required item
     local required = {}
     parentinfo = parentinfo or {}
@@ -212,9 +220,10 @@ end
 
 -- load package package from repositories
 function _load_package_from_repository(packagename, opt)
+    opt = opt or {}
     local packagedir, repo = repository.packagedir(packagename, opt)
     if packagedir then
-        return core_package.load_from_repository(packagename, repo, packagedir)
+        return core_package.load_from_repository(packagename, packagedir, {plat = opt.plat, arch = opt.arch, repo = repo})
     end
 end
 
@@ -329,6 +338,8 @@ function _add_package_configurations(package)
     package:add("configs", "cxflags", {builtin = true, description = "Set the C/C++ compiler flags."})
     package:add("configs", "cxxflags", {builtin = true, description = "Set the C++ compiler flags."})
     package:add("configs", "asflags", {builtin = true, description = "Set the assembler flags."})
+    package:add("configs", "ldflags", {builtin = true, description = "Set the binary linker flags."})
+    package:add("configs", "shflags", {builtin = true, description = "Set the shared library linker flags."})
 end
 
 -- select package version
@@ -360,10 +371,13 @@ function _select_package_version(package, requireinfo, locked_requireinfo)
     local version = nil
     local require_version = requireinfo.version
     local require_verify  = requireinfo.verify
-    if (not package:get("versions") or require_verify == false) and semver.is_valid(require_version) then
+    if (not package:get("versions") or require_verify == false)
+        and (semver.is_valid(require_version) or semver.is_valid_range(require_version)) then
         -- no version list in package() or need not verify sha256sum? try selecting this version directly
-        -- @see https://github.com/xmake-io/xmake/issues/930
+        -- @see
+        -- https://github.com/xmake-io/xmake/issues/930
         -- https://github.com/xmake-io/xmake/issues/1009
+        -- https://github.com/xmake-io/xmake/issues/3551
         version = require_version
         source = "version"
     elseif #package:versions() > 0 then -- select version?
@@ -495,15 +509,25 @@ end
 
 -- finish requireinfo
 function _finish_requireinfo(requireinfo, package)
+    -- we need to synchronise the plat/arch inherited from the parent package as early as possible
+    if requireinfo.plat then
+        package:plat_set(requireinfo.plat)
+    end
+    if requireinfo.arch then
+        package:arch_set(requireinfo.arch)
+    end
     requireinfo.configs = requireinfo.configs or {}
     if not package:is_headeronly() then
         if requireinfo.configs.vs_runtime == nil and package:is_plat("windows") then
             requireinfo.configs.vs_runtime = "MT"
         end
     end
-    -- we need ensure readonly configs
+    -- we need to ensure readonly configs
     for _, name in ipairs(table.keys(requireinfo.configs)) do
-        if package:extraconf("configs", name, "readonly") then
+        local current = requireinfo.configs[name]
+        local default = package:extraconf("configs", name, "default")
+        if package:extraconf("configs", name, "readonly") and current ~= default then
+            wprint("configs.%s is readonly in package(%s), it's always %s", name, package:name(), default)
             -- package:config() will use default value after loading package
             requireinfo.configs[name] = nil
         end
@@ -538,7 +562,7 @@ function _merge_requireinfo(requireinfo, requirepath)
         end
     end
 
-    -- append requireconf_extra into requireinfo
+    -- Append requireconf_extra into requireinfo
     -- and the configs of add_requires have a higher priority than add_requireconfs.
     --
     -- e.g.
@@ -547,16 +571,20 @@ function _merge_requireinfo(requireinfo, requirepath)
     --
     -- foo and bar will be debug mode
     --
-    -- we can also override the configs of add_requires
+    -- We can also override the configs of add_requires
     --
     -- e.g.
     -- add_requires("zlib 1.2.11")
     -- add_requireconfs("zlib", {override = true, version = "1.2.10"})
     --
-    -- we override the version of zlib to 1.2.10
+    -- We override the version of zlib to 1.2.10
     --
-    if #requireconf_result == 1 then
-        local requireconf_extra = requireconf_result[1].requireconf_extra
+    -- If the same dependency is matched to multiple configurations,
+    -- the configurations are merged by default,
+    -- and if override is set, then it rewrites the previous configurations.
+    --
+    for _, item in ipairs(requireconf_result) do
+        local requireconf_extra = item.requireconf_extra
         if requireconf_extra then
             -- preprocess requireconf_extra, (debug, override ..)
             local override = requireconf_extra.override
@@ -580,12 +608,6 @@ function _merge_requireinfo(requireinfo, requirepath)
                 end
             end
         end
-    elseif #requireconf_result > 1 then
-        local confs = {}
-        for _, item in ipairs(requireconf_result) do
-            table.insert(confs, item.requireconf)
-        end
-        raise("package(%s) will match multiple add_requireconfs(%s)!", requirepath, table.concat(confs, " "))
     end
 end
 
@@ -750,7 +772,10 @@ function _load_package(packagename, requireinfo, opt)
     local from_repo = false
     if not package then
         package = _load_package_from_repository(packagename, {
-            name = requireinfo.reponame, locked_repo = locked_requireinfo and locked_requireinfo.repo})
+            plat = requireinfo.plat,
+            arch = requireinfo.arch,
+            name = requireinfo.reponame,
+            locked_repo = locked_requireinfo and locked_requireinfo.repo})
         if package then
             from_repo = true
         end
@@ -804,6 +829,11 @@ function _load_package(packagename, requireinfo, opt)
         if requireinfo.is_toplevel and not package_cached:is_toplevel() then
             package_cached:requireinfo().is_toplevel = true
         end
+        -- mark this cached package.requireinfo as override
+        -- @see https://github.com/xmake-io/xmake/issues/4078
+        if requireinfo.override then
+            package_cached:requireinfo().override = true
+        end
         return package_cached
     end
 
@@ -834,7 +864,7 @@ function _load_package(packagename, requireinfo, opt)
     -- check package configurations
     _check_package_configurations(package)
 
-    -- save artifacts info, we need add it at last before buildhash need depend on package configurations
+    -- save artifacts info, we need to add it at last before buildhash need depend on package configurations
     -- it will switch to install precompiled binary package from xmake-mirror/build-artifacts
     if from_repo and not option.get("build") and not requireinfo.build then
         local artifacts_manifest = repository.artifacts_manifest(packagename, version)
@@ -977,17 +1007,15 @@ function _compatible_with_previous_librarydeps(package, opt)
         return true
     end
 
-    -- has been checked?
-    local compatible_checked = package:data("librarydeps.compatible_checked")
-    if compatible_checked then
-        return
-    end
-
     -- check strict compatibility for librarydeps?
     local strict_compatibility = project.policy("package.librarydeps.strict_compatibility")
     if strict_compatibility == nil then
         strict_compatibility = package:policy("package.librarydeps.strict_compatibility")
     end
+    -- and we can disable it manually, @see https://github.com/xmake-io/xmake/pull/3738
+    if strict_compatibility == false then
+       return true
+    end
 
     -- compute the buildhash for current librarydeps
     local depnames = hashset.new()
@@ -1068,7 +1096,7 @@ function should_install(package, opt)
     if package:exists() and _compatible_with_previous_librarydeps(package, opt) then
         return false
     end
-    -- we need not install it if this package need only be fetched
+    -- we don't need to install it if this package only need to be fetched
     if package:is_fetchonly() then
         return false
     end
